<?php

declare(strict_types=1);

namespace Drupal\known_overrides\Controller;

use Drupal\Core\Site\Settings;
use Drupal\Core\Config\ConfigManagerInterface;
use Drupal\Core\Config\StorageCacheInterface;
use Drupal\Core\Controller\ControllerBase;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Config\ConfigFactoryInterface;

/**
 * Returns responses for Known Overrides routes.
 */
final class KnownOverridesController extends ControllerBase {

  /**
   * Config Factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The controller constructor.
   */
  public function __construct(
    protected readonly Settings $settings,
    private readonly StorageCacheInterface $configStorage,
    private readonly ConfigManagerInterface $configManager,
    ConfigFactoryInterface $configFactory,
  ) {
    $this->configFactory = $configFactory;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container): self {
    return new self(
      $container->get('settings'),
      $container->get('config.storage'),
      $container->get('config.manager'),
      $container->get('config.factory')
    );
  }

  /**
   * Builds the response.
   */
  public function __invoke(): array {
    // Access the knownOverrides array from settings.php.
    $knownOverrides = $this->settings->get('knownOverrides', []);
    // Reduce list down to unique values.
    $unique = [];
    foreach ($knownOverrides as $configName) {
      $unique[$configName] = $configName;
    }
    // Re-index the array.
    $knownOverrides = array_values($unique);
    $build = [
      '#theme' => 'known_overrides_report',
      '#template' => 'known_overrides_report',
      '#report' => [],
      '#overrides' => count($knownOverrides),

    ];
    /*
     * Loop through the knownOverrides array and compare
     * the editable and memory/potential overrides.
     * Return the differences to the template.
     */
    foreach ($knownOverrides as $configName) {
      $configEditable = $this->configManager->getConfigFactory()
        ->getEditable($configName);
      $editableList = (array) ($configEditable->get());
      $configList = (array) ($this->configFactory->get($configName)->get());

      $diff = $this->compareArraysRecursive($editableList, $configList);
      $diff['only_in_editable'] = json_encode((array) $diff['only_in_editable'], JSON_PRETTY_PRINT);
      $diff['only_in_memory'] = json_encode((array) $diff['only_in_memory'], JSON_PRETTY_PRINT);

      $build['#report'][$configName] = $diff;
    }

    return $build;
  }

  /**
   * Compare two arrays recursively and return the differences.
   *
   * @param array $editableArray
   *   The compare date from the editable config.
   * @param array $memoryArray
   *   The compare data from the memory config.
   *
   * @return array|array[]
   *   Return any differences between the two arrays.
   */
  private function compareArraysRecursive(array $editableArray, array $memoryArray) {
    $differences = [
      'only_in_editable' => [],
      'only_in_memory' => [],
    ];

    // Compare values in array1 that are not in array2.
    foreach ($editableArray as $key => $value) {
      if (array_key_exists($key, $memoryArray)) {
        if (is_array($value) && is_array($memoryArray[$key])) {
          // Recursive comparison for nested arrays.
          $nested_diff = $this->compareArraysRecursive($value, $memoryArray[$key]);
          if (!empty($nested_diff['only_in_editable']) || !empty($nested_diff['only_in_memory'])) {
            $differences['only_in_editable'][$key] = $nested_diff['only_in_editable'];
            $differences['only_in_memory'][$key] = $nested_diff['only_in_memory'];
          }
        }
        elseif ($value !== $memoryArray[$key]) {
          // Different values for the same key.
          $differences['only_in_editable'][$key] = $value;
          $differences['only_in_memory'][$key] = $memoryArray[$key];
        }
      }
      else {
        // Key exists only in array1.
        $differences['only_in_editable'][$key] = $value;
      }
    }

    // Compare values in array2 that are not in array1.
    foreach ($memoryArray as $key => $value) {
      if (!array_key_exists($key, $editableArray)) {
        $differences['only_in_memory'][$key] = $value;
      }
    }

    return $differences;
  }

}
